unit Main;

interface

uses
  SysUtils, Windows, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls, Menus, Spin,
  OverByteIcsWSocket, OverByteIcsWSocketS, System.UITypes;

type
  TMainForm = class(TForm)
    Memo1: TMemo;
    Panel1: TPanel;
    Label1: TLabel;
    ListenBtn: TButton;
    PortEdit: TSpinEdit;
    ClearBtn: TButton;
    SendBtn: TButton;
    procedure FormCreate(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure ListenBtnClick(Sender: TObject);
    procedure SendBtnClick(Sender: TObject);

    private
     Server: TWSocketServer;  // Listening Socket
     Buffer: ansistring;
     PCount: integer;

    procedure ServerClientConnect(Sender: TObject; Client: TWSocketClient;
      Error: Word);
    procedure ServerDataAvailable(Sender: TObject; Error: Word);
    procedure ParsePacket(const p: ansistring);
  end;


var
  MainForm: TMainForm;

implementation

{$R *.DFM}

 const
   HeaderLen = 24;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Server := TWSocketServer.Create( Self );
  Server.OnClientConnect := ServerClientConnect;
  Server.Banner := '';
end;

procedure TMainForm.ClearBtnClick(Sender: TObject);
begin
  Memo1.Lines.Clear;
end;

procedure TMainForm.ListenBtnClick(Sender: TObject);
begin
  // Preparing the server socket for listening
  Server.Port := PortEdit.Text;
  Server.Addr := '0.0.0.0';
  Server.Listen;
end;

procedure TMainForm.ServerClientConnect(Sender: TObject;
  Client: TWSocketClient; Error: Word);
begin
  // Every connection will call our event handler on new data arrival
  Client.OnDataAvailable := ServerDataAvailable;
  Memo1.Lines.Add( 'Accepted connection from ' + Client.PeerAddr );
end;

procedure TMainForm.ServerDataAvailable(Sender: TObject; Error: Word);
var
  BufLen: integer;
  packet: ansistring;
  PLen: word;
begin
  if Error = 0 then
  begin
    //Append receieved data to the buffer
    Buffer:= Buffer + ( Sender as TWSocket ).ReceiveStrA;
    repeat
      PLen := 0;
      BufLen := Length(Buffer);
      //Read the expected packet length from the header
      if BufLen > SizeOf(word) then Move(Buffer[1], PLen, SizeOf(word));
      //If we could read the packet length value *and* the entire packet is already
      //in the buffer, let's parse it and remove it from the buffer
      if (PLen > 0) and (BufLen >= PLen + HeaderLen) then
      begin
        //Copy the packet and the header
        Packet:=Copy(Buffer, 1, PLen + HeaderLen);
        //Delete the packet and the header from the buffer
        Delete(Buffer, 1, PLen + HeaderLen);
        //Parse the packet
        ParsePacket(packet);
      end
        else break;  //the buffer doesn't contain enough data or we've processed all data, so we just exit
    until false;
  end;
end;

procedure TMainForm.ParsePacket(const p: ansistring);
var
  Dir, Data: string;
begin
  Inc(PCount);

  //Detect the packet direction
  case Byte(p[22]) of   //The packet direction is stored in the 22nd byte of the header 
    0: Dir := 'pass-through';
    1: Dir := 'inbound';
    2: Dir := 'outbound';
    else raise Exception.Create( 'Wrong direction' );
  end;

  //Now get a few bytes of sample data to show in the memo window
  Data:=IntToHex(byte(p[HeaderLen + 1]), 2) +
        IntToHex(byte(p[HeaderLen + 2]), 2) +
        IntToHex(byte(p[HeaderLen + 3]), 2) +
        IntToHex(byte(p[HeaderLen + 4]), 2);

  //Display the packet
  Memo1.Lines.Add( Format( 'Packet #%d, Size: %d, Direction: %s, Data: 0x%s ...',
                                 [PCount, length(p)- HeaderLen, Dir, Data]));
end;

procedure TMainForm.SendBtnClick(Sender: TObject);
var
  PacketData, Msg: ansistring;
  Len, RateIndex: word;
begin
  if MessageDlg('Are you sure?', mtConfirmation, [mbYes, mbNo], 0) <> mrYes then exit;
  if (Server.ClientCount = 1) and (Server.Client[0].State = wsConnected) then
  begin
    Msg := 'Sample packet text. You shouldn''t send this in real life!';
    Len := Length(Msg);
    //Make space for the packet and the four-byte header
    SetLength(PacketData, Len + 4);
    //Copy the packet starting from the 5th byte
    Move(Msg[1], PacketData[5], Len);
    //Write the packet length
    Move(Len, PacketData[1], SizeOf(word));
    //Write the rate index. Make it 2, for example.
    RateIndex := 2;
    Move(RateIndex, PacketData[3], SizeOf(word));
    //Send
    Server.Client[0].SendStr(PacketData);
  end
    else ShowMessage('Not connected');
end;

end.
